package CVSMonitor::Plugin;

# This module provides the beginnings of plug-in support for CVSMonitor,
# which will mainly be used to support various file type viewers.

use strict;
use UNIVERSAL 'isa';
use File::Find::Rule;

# Are we loaded?
use vars qw{$loaded};
BEGIN {
	$loaded = {
		general => 0,
		viewer => 0,
		};
}

# Initialises the Plug-in subsystem.
# We are passed a copy of the config.
sub load {
	my $class = shift;
	my $Config = isa( $_[0], 'CVSMonitor::Config' )
		? shift : return undef;
	my $type = shift || 'all';
	return 1 if $loaded->{$type};
	
	# Get the plug-ins
	my $plugins = $Config->{plugins} || {};
	
	# Load each of the plugins
	foreach my $module ( keys %$plugins ) {
		# Skip if the type is loaded, or not wanted
		next unless ( $type eq $plugins->{$module} or $type eq 'all' );
		next if $loaded->{$plugins->{$module}};
		
		# Load the class
		unless ( Class::Autouse->load( $module ) ) {
			return $class->andError( "Error loading plug-in module '$module'" );
		}
		
		# Call the register method for the plug-in to register itself.
		unless ( $module->register() ) {
			return $class->andError( "Plug-in module '$module' returned an error while registering" );
		}		
	}
	
	# Mark the type as loaded
	if ( $type eq 'all' ) {
		$loaded->{general} = 1;
		$loaded->{viewer} = 1;
	} else {
		$loaded->{$type} = 1;
	}
}

1;
