package CVSMonitor::Range::Basics;

# Provides an implementation of a set of basic numeric values.

use strict;
use UNIVERSAL qw{isa can};
use base 'CVSMonitor::Range';

use vars qw{@field_list};
BEGIN {
	@field_list = qw{add remove increase decrease karma};
}



#####################################################################
# Functional Methods

# Add Version information to the Range totals
sub add {
	my $self = shift;
	my $Source = shift or return undef;
	my $balanced = shift;
	return undef unless can( $Source, 'getChangeSets' );
	my $ChangeSets = $Source->getChangeSets or return undef;
	
	# Index the dates
	my %totals = ();
	$totals{$_->{date}} = $_ foreach @$self;

	# Work out what level of depth to use
	my $regex = CVSMonitor::DateDepth->isolate_regex( $self->depth ) or return undef;

	# Iterate over the change sets
	foreach my $ChangeSet ( @$ChangeSets ) {
		return undef unless $ChangeSet->date =~ $regex;
		next unless exists $totals{$1};
		my $key = $1;		

		# Add to the changes totals
		$totals{$key}->{commits}++;
		$totals{$key}->{changes} += $ChangeSet->changes;
		
		# Get the line information
		my $lines = $balanced
			? $ChangeSet->getBalancedLines
			: $ChangeSet->getLines;
		foreach ( @field_list ) {
			next unless defined $totals{$key}->{$_};
			$totals{$key}->{$_} += $lines->{$_};
		}
	}

	return 1;
}	
	
# Get a hash of the totals for the Range
sub totals {
	my $self = shift;
	
	my $totals = {
		commits  => 0,
		changes  => 0,
		
		add      => 0,
		remove   => 0,
		increase => 0,
		decrease => 0,
		
		karma   => 0,
		};
		
	# Add up the dates
	foreach my $date ( @$self ) {
		foreach ( keys %$totals ) {
			$totals->{$_} += $date->{$_};
		}
	}
	
	# Add the extra stuff
	$totals->{up} = $totals->{add} + $totals->{increase};
	$totals->{down} = $totals->{remove} + $totals->{decrease};
	$totals->{activity} = $totals->{up} + $totals->{down};
	$totals->{change} = $totals->{up} - $totals->{down};
	
	return $totals;
}

# For a given date, generate it's basic stats struct
sub getStatisticsStruct {
	my $either = shift;
	my $date = shift;
	return undef unless isa( $date, 'CVSMonitor::DateDepth' );
	
	return {
		date => $date->toString,
		commits => 0,
		changes => 0,
		add => 0, 
		remove => 0,
		increase => 0,
		decrease => 0,
		karma => 0,
		};	
}

1;

__END__

# Copyright (c) 2002 Adam Kennedy. All rights reserved.
# This program is free software; you can redistribute
# it and/or modify it under the same terms as Perl itself.
#
# The full text of the license can be found in the
# LICENSE.PERL file included with this module.
