package CVSMonitor::Range::Files;

use strict;
use UNIVERSAL qw{isa can};
use base 'CVSMonitor::Range';

# Add Version information to the Range totals
sub add {
	my $self = shift;
	my $Source = shift or return undef;
	my $balanced = shift;
	return undef unless can( $Source, 'getChangeSets' );
	my $ChangeSets = $Source->getChangeSets or return undef;
	
	# Index the dates
	my %totals = ();
	$totals{$_->{date}} = $_ foreach @$self;

	# Work out what level of depth to use
	my $regex = CVSMonitor::DateDepth->isolate_regex( $self->depth ) or return undef;

	# Iterate over the change sets
	foreach my $ChangeSet ( @$ChangeSets ) {
		return undef unless $ChangeSet->date =~ $regex;
		next unless exists $totals{$1};
		my $key = $1;		

		foreach my $Version ( $ChangeSet->getVersions ) {
			if ( $Version->created ) {
				$totals{$key}->{added}++;
			} elsif ( $Version->removed ) {
				$totals{$key}->{removed}++;
			}	
		}
	}

	return 1;
}

# Get a hash of the totals for the Range
sub totals {
	my $self = shift;
	
	my $totals = {
		added => 0,
		removed => 0,
		};
		
	# Add up the dates
	foreach my $date ( @$self ) {
		$totals->{added} += $date->{added};
		$totals->{removed} += $date->{removed};
	}
	
	# Add the extra stuff
	$totals->{change} = $totals->{added} - $totals->{removed};
	
	return $totals;
}

# For a given date, generate it's basic stats struct
sub getStatisticsStruct {
	my $either = shift;
	my $date = shift;
	return undef unless isa( $date, 'CVSMonitor::DateDepth' );
	
	return {
		date => $date->toString,
		added => 0,
		removed => 0,
		};	
}

1;
 
__END__

# Copyright (c) 2002 Adam Kennedy. All rights reserved.
# This program is free software; you can redistribute
# it and/or modify it under the same terms as Perl itself.
#
# The full text of the license can be found in the
# LICENSE.PERL file included with this module.
