package CVSMonitor::VersionNumber;

# This package implements some math for deep version numbers,
# as they exist in CVS.

use strict;
use UNIVERSAL 'isa';




#####################################################################
# Constructor and Friends

sub new {
	my $class = shift;
	my $revision = shift;
	
	# Check the version
	return undef unless length $revision;
	return undef unless $revision =~ /^[\d.]+$/;
	
	# Get the revision parts
	my $parts = [ split /\./, $revision ];
	
	# Create and retunr the object
	return bless $parts, $class;
}

# Create a copy of ourself
sub copy {
	my $self = shift;
	return undef unless ref $self;
	my $class = ref $self;
	
	my $copy = [ $self->parts ];
	return bless $copy, $class;
}





#####################################################################
# Access Methods

sub revision { join '.', @{$_[0]} } 
sub parts { @{$_[0]} }
sub elements { scalar @{$_[0]} }





#####################################################################
# Main Methods

# Find the next revision
sub next {
	# Get the object and return type
	my $either = shift;
	my $return_object = 0;
	my $self = undef;
	if ( isa( $self, 'CVSMonitor::VersionNumber' ) ) {
		$return_object = 1;
		$self = $either->copy; # Always work on a copy
	} else {
		$self = $either->new( shift ) or return undef;
	}

	# Add 1 to the last part of the copy
	$self->[ $#$self ]++;
	
	# Return the object or string version as needed
	return $return_object ? $self : $self->revision;
}

# Get the previous revision
sub previous {
	# Get the object and return type
	my $either = shift;
	my $return_object = 0;
	my $self = undef;
	if ( isa( $self, 'CVSMonitor::VersionNumber' ) ) {
		$return_object = 1;
		$self = $either->copy; # Always work on a copy
	} else {
		$self = $either->new( shift ) or return undef;
	}

	# Calculate the previous version
	if ( $self->[-1] == 1 ) {
		# If we are the first possible version, return 0
		if ( scalar @$self == 2 ) {
			return 0;
		}
		
		# Remove the last two version increments
		pop @$self; pop @$self;
	} else {
		$self->[ $#$self ]--;
	}	

	# Return the object or string version as needed
	return $return_object ? $self : $self->revision;	
}

1;
