package setup::Default;

# The setup::Default package allows a class that inherits from
# it to act as both an instantiatable and static class.
# Whenever a static method call is made, the method is applied
# instead to a cache default instantiation of the class, in effect
# somewhat like a singleton class. However, setup::Default still allows
# you to instantiate additional objects if you want.

use strict;
use UNIVERSAL 'isa';

# Create the default object storage
use vars qw{%default};
BEGIN { 
	%default = ();
}

# Retrieve the default object for a class
sub default {
	unless ( $default{$_[0]} ) {
		$default{$_[0]} = $_[0]->new;
	}
	return $default{$_[0]};
}

# Get the default object if we are passed the class name
sub self { ref $_[0] ? $_[0] : $_[0]->default }

# Provide a default Constructor.
# This should be overloaded
sub new { bless {}, $_[0] }

# Explicitly initialise the default object.
# Will overwrite an existing default.
# @returns Newly created default on success
# @returns Return value of ->new on error
sub initDefault {
	my $class = shift;
	my $newObject = $class->new( @_ );
	return $newObject unless isa( $newObject, $class );
	$default{$class} = $newObject;
	return $newObject;
}	
	
1;
