package setup::Tests;

# This package contains the standard test battery we use.

use strict;
use UNIVERSAL 'isa';
use ExtUtils::MakeMaker;

use vars qw{@ISA};
BEGIN { @ISA = 'setup::Error' }





#####################################################################
# Tests, loosely grouped by section

# Is the version of perl new enough
sub perlVersion {
	my $class = shift;
	my $version = shift;
	
	return ( $] > $version ) ? 1 : 0;
}

# Is a module installed
sub perlModule {
	my $class = shift;
	my $module = shift;
	return 1 unless $module;

	# Can we find a file for this module
	my $file = setup::Autouse::_class_file( $module ) or return 0;
	$file = setup::Autouse::_file_exists( $file ) or return 0;

	# Return the version of the file, if we can tell
	return MM->parse_version( $file ) || -1;
}

# Tests if something will actually be called when we
# try to run a certain program.
sub programInstalled {
	my $class = shift;
	my $program = shift or return 1;
	
	my $which = setup::Which::which( $program );
	return defined $which ? 1 : 0;	
}

# Does a file exist
sub filesystemExists {
	my $class = shift;
	my $location = shift;
	return -e $location ? 1 : 0
}

# ... and can we read it
sub filesystemCanRead {
	my $class = shift;
	my $location = shift;
	return 0 unless -e $location;
	if ( -d $location ) {
		return (-x $location and -r $location) ? 1 : 0;
	} else {
		return (-r $location) ? 1 : 0;
	}
}

# ... or write it
sub filesystemCanWrite { 
	my $class = shift;
	my $location = shift;

	return 0 unless -e $location;
	if ( -d $location ) {
		return (-x $location and -w $location) ? 1 : 0;
	} else {
		return (-w $location) ? 1 : 0;
	}

}

# ... or both
sub filesystemCanReadWrite { 
	my $class = shift;
	my $location = shift;
	
	return 0 unless -e $location;
	if ( -d $location ) {
		return (-x $location and -r $location and -w $location) ? 1 : 0;
	} else {
		return (-r $location and -w $location) ? 1 : 0;
	}

}

# ... or can we execute it
sub filesystemCanExecute {
	my $class = shift;
	my $location = shift;
	
	return 0 unless -e $location;
	if ( -d $location ) {
		return (-x $location) ? 1 : 0;
	} else {
		return (-r $location and -x $location) ? 1 : 0;
	}
}

1;
